/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * LRSpectra.hpp
 *
 *  Created on: 2010/11/19
 *      Author: HARK Development Team
 */

#ifndef LRSIGNAL_HPP_
#define LRSIGNAL_HPP_

#include <Channel.hpp>
#include <Signal.hpp>

namespace FD
{
template <typename T> class RCPtr;
class Object;
typedef RCPtr<Object> ObjectRef;
template <class _Key, class _Tp> class Map;
}

namespace NSHarkBinaural
{

/**
 * 左右両チャンネルの周波数領域信号を表す型
 */
typedef ChannelHolder<Spectrum::ptr_t, Spectrum::ptr_t> LRSpectra;

/**
 * 左右両チャンネルの時間領域信号を表す型
 */
typedef ChannelHolder<Unspectrum::ptr_t, Unspectrum::ptr_t> LRUnspectra;

/**
 * 左右の信号の要素数が等しいかどうか.
 * 左右の要素を信号とするChannelHolder型のインスタンスを受け取り、
 * 左右の信号の要素数が一致するか検査する。
 * holder、holder->Left()、holder->Right()のいずれかがnull参照の場合または
 * 左右の信号の要素数が一致しない場合はfalseを返す。
 * @param holder 左右の要素を信号とするデータの組
 * @return 左右の信号の要素数が等しければtrue
 */
template <typename T>
//bool IsEqualSignalSize(typename ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>::ptr_t holder)
//bool IsEqualSignalSize(typename ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>::ptr_t holder)
bool IsEqualSignalSize(boost::shared_ptr<ChannelHolder<boost::shared_ptr<SignalBase<T> >, boost::shared_ptr<SignalBase<T> > > > holder)
{
    return holder.get() != 0 && holder->Left().get() != 0 && holder->Right().get() != 0 &&
           holder->Left()->Size() == holder->Right()->Size();
}

/**
 * 2チャンネル信号をモノラル信号へミックスする.
 * @param lrsignal 2チャンネル信号
 * @return ミックスされた信号
 */
template <typename T>
typename SignalBase<T>::ptr_t MixToMonaural(
    typename ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>::ptr_t lrsignal)
{
    typename SignalBase<T>::ptr_t result;
    if (IsEqualSignalSize(lrsignal))
    {
        size_t size = lrsignal->Left()->Size();
        result = SignalBase<T>::ptr_t(new SignalBase<T>(size));
        for (size_t i = 0; i < size; ++i)
        {
            result->At(i) = (lrsignal->Left()->At(i) + lrsignal->Right()->At(i)) / 2;
        }
    }
}

/**
 * 空の左右信号を生成する.
 * @tparam T 生成する信号の要素型
 */
template <typename T>
class LRSignalFactory
{
public:

    /**
     * 指定された要素数をもつ、空の左右信号を生成する.
     * @param n 要素数
     */
    static typename ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>::ptr_t
    Create(size_t n)
    {
        typename SignalBase<T>::ptr_t l(new SignalBase<T>(n));
        typename SignalBase<T>::ptr_t r(new SignalBase<T>(n));
        if (l.get() == 0 || r.get() == 0)
        {
            return typename ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>::ptr_t();
        }
        return typename ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>::ptr_t(
                   new ChannelHolder<typename SignalBase<T>::ptr_t, typename SignalBase<T>::ptr_t>(l, r));
    }
};

}

#endif /* LRSIGNAL_HPP_ */
